#include "main.h"

// Include the generated source index.
// NOTE: Keep the project name and this path in sync.
#include "Generated/ReflectionVisitors/Generated.ocppindex"
#include <conio.h>
#include <iostream>

using std::cout;
using std::endl;

/*=== visitors ===*/

using opcpp::base::visitor_base;
using namespace opcpp::accessors;

class num_bytes_visitor : public visitor_base
{
private:

	size_t NumBytes;

public:

	num_bytes_visitor()
		: NumBytes(0)
	{

	}

	size_t GetNumBytes() const
	{
		return NumBytes;
	}

	void visit(member_info& info)
	{
		NumBytes += info.type_size();
	}

	void visit(string_info& info)
	{
		NumBytes += info.type_size();
		NumBytes += info.get_string().length() * sizeof( char );
	}

	void visit(class_info& info)
	{
		info.visit_data_members( *this );
	}

	void visit(container_info& info)
	{
		NumBytes += info.type_size();
		
		for (bool i = info.begin(); i; i = info.increment())
			info.get_inner()->visit( *this );
	}
};

// Entry point into the application.
int main()
{
	BankAccount       account;
	num_bytes_visitor visitor;

	account.Payees.push_back( Payee() );
	account.Payees.push_back( Payee() );
	account.Transactions.push_back( 13.30 );
	
	account.visit_data_members( visitor );

	cout << "BankAccount (" << visitor.GetNumBytes() << " bytes)" << endl;

	_getch();
	
	return 0;
}